==> symbol-processing-aa-embeddable-2.2.0-2.0.2-sources.jar: org/jetbrains/kotlin/fir/utils/exceptions/firExceptionUtils.kt <==
/*
 * Copyright 2010-2024 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.utils.exceptions

import ksp.org.jetbrains.kotlin.KtSourceElement
import ksp.org.jetbrains.kotlin.fir.FirElement
import ksp.org.jetbrains.kotlin.fir.FirElementWithResolveState
import ksp.org.jetbrains.kotlin.fir.FirModuleData
import ksp.org.jetbrains.kotlin.fir.renderer.*
import ksp.org.jetbrains.kotlin.fir.types.ConeClassifierLookupTag
import ksp.org.jetbrains.kotlin.fir.symbols.FirBasedSymbol
import ksp.org.jetbrains.kotlin.fir.symbols.impl.FirCallableSymbol
import ksp.org.jetbrains.kotlin.fir.symbols.impl.FirClassifierSymbol
import ksp.org.jetbrains.kotlin.fir.types.ConeClassLikeLookupTag
import ksp.org.jetbrains.kotlin.fir.types.ConeKotlinType
import ksp.org.jetbrains.kotlin.utils.exceptions.ExceptionAttachmentBuilder

fun ExceptionAttachmentBuilder.withFirEntry(name: String, fir: FirElement) {
    withEntry(name, fir) {
        FirRenderer(
            resolvePhaseRenderer = FirResolvePhaseRenderer(),
            declarationRenderer = FirDeclarationRendererWithAttributes(),
        ).renderElementAsString(it)
    }
    withEntry("${name}ElementKind", fir.source?.kind?.let { it::class.simpleName })
    if (fir is FirElementWithResolveState) {
        withModuleDataEntry("${name}ModuleData", fir.moduleData)
    }
    withSourceEntry("${name}Source", fir.source)
}

fun ExceptionAttachmentBuilder.withFirSymbolIdEntry(name: String, symbol: FirBasedSymbol<*>?) {
    when (symbol) {
        is FirClassifierSymbol -> withFirLookupTagEntry(name, symbol.toLookupTag())
        is FirCallableSymbol -> withEntry(name, symbol.callableId.toString())
        else -> withEntry(name, symbol.toString())
    }
}

fun ExceptionAttachmentBuilder.withFirLookupTagEntry(name: String, lookupTag: ConeClassifierLookupTag?) {
    withEntry(name, lookupTag) { tag ->
        when (tag) {
            is ConeClassLikeLookupTag -> tag.classId.asString()
            else -> tag.name.asString()
        }
    }
}

fun ExceptionAttachmentBuilder.withSourceEntry(name: String, source: KtSourceElement?) {
    withEntry(name, source) { it.getElementTextInContextForDebug() }
}

fun ExceptionAttachmentBuilder.withModuleDataEntry(name: String, moduleData: FirModuleData?) {
    withEntry(name, moduleData) { module ->
        buildString {
            append("Name: ${module.name}, ")
            append("Platform: ${module.platform}")
        }
    }
}

fun ExceptionAttachmentBuilder.withFirSymbolEntry(name: String, symbol: FirBasedSymbol<*>) {
    withFirEntry("${name}Fir", symbol.fir)
}

fun ExceptionAttachmentBuilder.withConeTypeEntry(name: String, coneType: ConeKotlinType?) {
    withEntry(name, coneType) {
        buildString { ConeTypeRendererForDebugging(this).render(it) }
    }
}

==> symbol-processing-aa-embeddable-2.3.4-sources.jar: org/jetbrains/kotlin/fir/utils/exceptions/firExceptionUtils.kt <==
/*
 * Copyright 2010-2024 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.utils.exceptions

import ksp.org.jetbrains.kotlin.KtSourceElement
import ksp.org.jetbrains.kotlin.fir.FirElement
import ksp.org.jetbrains.kotlin.fir.FirElementWithResolveState
import ksp.org.jetbrains.kotlin.fir.FirModuleData
import ksp.org.jetbrains.kotlin.fir.renderer.*
import ksp.org.jetbrains.kotlin.fir.types.ConeClassifierLookupTag
import ksp.org.jetbrains.kotlin.fir.symbols.FirBasedSymbol
import ksp.org.jetbrains.kotlin.fir.symbols.impl.FirCallableSymbol
import ksp.org.jetbrains.kotlin.fir.symbols.impl.FirClassifierSymbol
import ksp.org.jetbrains.kotlin.fir.types.ConeClassLikeLookupTag
import ksp.org.jetbrains.kotlin.fir.types.ConeKotlinType
import ksp.org.jetbrains.kotlin.utils.exceptions.ExceptionAttachmentBuilder

fun ExceptionAttachmentBuilder.withFirEntry(name: String, fir: FirElement?) {
    withEntry(name, fir) { fir ->
        FirRenderer(
            resolvePhaseRenderer = FirResolvePhaseRenderer(),
            declarationRenderer = FirDeclarationRendererWithAttributes(),
        ).renderElementAsString(fir)
    }

    if (fir != null) {
        withEntry("${name}ElementKind", fir.source?.kind?.let { it::class.simpleName })
        if (fir is FirElementWithResolveState) {
            withModuleDataEntry("${name}ModuleData", fir.moduleData)
        }
        withSourceEntry("${name}Source", fir.source)
    }
}

fun ExceptionAttachmentBuilder.withFirSymbolIdEntry(name: String, symbol: FirBasedSymbol<*>?) {
    when (symbol) {
        is FirClassifierSymbol -> withFirLookupTagEntry(name, symbol.toLookupTag())
        is FirCallableSymbol -> withEntry(name, symbol.callableId.toString())
        else -> withEntry(name, symbol.toString())
    }
}

fun ExceptionAttachmentBuilder.withFirLookupTagEntry(name: String, lookupTag: ConeClassifierLookupTag?) {
    withEntry(name, lookupTag) { tag ->
        when (tag) {
            is ConeClassLikeLookupTag -> tag.classId.asString()
            else -> tag.name.asString()
        }
    }
}

fun ExceptionAttachmentBuilder.withSourceEntry(name: String, source: KtSourceElement?) {
    withEntry(name, source) { it.getElementTextInContextForDebug() }
}

fun ExceptionAttachmentBuilder.withModuleDataEntry(name: String, moduleData: FirModuleData?) {
    withEntry(name, moduleData) { module ->
        buildString {
            append("Name: ${module.name}, ")
            append("Platform: ${module.platform}")
        }
    }
}

fun ExceptionAttachmentBuilder.withFirSymbolEntry(name: String, symbol: FirBasedSymbol<*>) {
    withFirEntry("${name}Fir", symbol.fir)
}

fun ExceptionAttachmentBuilder.withConeTypeEntry(name: String, coneType: ConeKotlinType?) {
    withEntry(name, coneType) {
        buildString { ConeTypeRendererForDebugging(this).render(it) }
    }
}
